function delta = RIP_constant(Phi,k,ca,rdm)
% function delta = RIP_constant(Phi,k,ca,rdm)
% =========================================================================
%  FILE:           RIP_constant.m     
%  AUTHOR:         Andreas M. Tillmann, TU Braunschweig, Germany
% =========================================================================
%  DESCRIPTION:    Compute or approximate (lower bound) the (symmetric-)RIP 
%                  constant delta via Eigenvalue Range of Pseudo-Inverse on
%                  support sets of (up to) a given size k.
%                  (If (the exact) delta is < 1, Phi possesses the Restric-
%                   ted Isometry Property (RIP) and is said to obey the
%                   Uniform Uncertainty Principle (UUP), see Candes/Tao 
%                   2004, 2005.)
%
%                  Remark:
%                  To actually prove RIP, ALL support sets (i.e., subsets 
%                  of column indices of the input matrix) of size (<=) k 
%                  have to be tested, i.e., for (exponentially many) column
%                  index subsets one has to compute the largest and 
%                  smallest eigenvalue and check for compliance with a 
%                  computed constant. This may result in extreme running
%                  times already for small subset sizes.
%
%  INPUT:   Phi    The matrix under consideration
%           k      The support size
%           ca     Indicator variable; pass ca=1 to check ALL support sets
%                  of size <= k, ca=0 to check only those with size == k
%                  [ optional; default is ca = 1 ]
%           rdm    Pass rdm=0 to check all support sets (size/s specified
%                  by ca), or a value >0 to check rdm-many random column
%                  index subsets
%                  [ optional; default is rdm = 0 ]
%
%  OUTPUT:  delta  The (approximate) symmetric-RIP constant
%
%  USAGE EXAMPLES:
%  - To compute the exact RIP constant for a matrix A for support size 4:
%       delta = RIP_constant(A,4)
%       (equivalently, delta = RIP_constant(A,4,1,0) )
%  - To compute a lower bound on the RIP constant for matrix B and support
%    size 25, one might wish to consider 10,000 random column index 
%    subsets of size 25:
%       delta = RIP_constant(B,25,0,10000)
% =========================================================================
% $Revision: 213 $
% $Author: andtillm $
% $Date: 2011-07-13 17:44:42 +0200 (Mi, 13. Jul 2011) $
% =========================================================================
if nargin < 3
    ca = 1;
end
if nargin < 4
    rdm = 0;
end
if( rdm > 0 )
    % Initialize random number generator seed stream:
    RandStream.setDefaultStream(RandStream('mt19937ar','seed',sum(100*clock)));
end
n     = size(Phi,2);
delta = -inf;
if( ca )
    r = k;    
else % ca == 0
    r = 1:k;
end
for s = r
    if( rdm == 0 )
        T = nchoosek(1:n,s);
        for i = 1:size(T,1)
            e = eig(Phi(:,T(i,:))'*Phi(:,T(i,:)));
            mi = min(e);
            ma = max(e);
            mm = max(1-mi,ma-1);
            if( delta < mm )
                delta = mm;
                if( delta >= 1 ) % RIP cannot hold
                    delta = -inf;
                    disp('RIP invalid, constant is larger than 1');
                    return;
                end
            end
        end
    else % rdm > 0
        for j = 1:rdm
            T = randperm(n); T = T(1:s);
            e = eig(Phi(:,T)'*Phi(:,T));
            mi = min(e);
            ma = max(e);
            mm = max(1-mi,ma-1);
            if( delta < mm )
                delta = mm;
                if( delta >= 1 ) % RIP cannot hold
                    delta = -inf;
                    disp('RIP invalid, constant is larger than 1');
                    return;
                end
            end
        end
    end
end
