function X = pinvSp(A,varargin)
% function X = pinvSp(A,varargin)
%==========================================================================
%  FILE:           pinvSp.m
%  AUTHOR:         Andreas M. Tillmann, TU Braunschweig, Germany
%                  MATLAB (all parts of function PINV)
%==========================================================================
%
%  DESCRIPTION:     This is a modification of the MATLAB function PINV  
%                   that also accepts sparse-format input matrices.
%   
%                   The function computes the pseudo-inverse of the input
%                   matrix A. The computations rely on the singular value
%                   decompositon; singular values below a tolerance are
%                   treated as zero.
%                   For a bit more detailed description, see PINV.
%
%  INPUT:   A       The matrix for which the pseudo-inverse is to be com-
%                   puted
%           tol     Tolerance for thresholding small singular values
%                   [ optional; from PINV, default is
%                     tol = max(size(A))*norm(A)*eps(class(A)) ]
%  OUTPUT:  X       The pseudo-inverse of A
%==========================================================================
% $Revision: 121 $
% $Author: andtillm $
% $Date: 2011-04-26 14:54:45 +0200 (Di, 26. Apr 2011) $
% =========================================================================
if( isempty(A) ) % quick return
  X = zeros(size(A'),class(A));  
  return;
end
if( ~issparse(A) ) % use standard pinv function for non-sparse A
  X = pinv(A,varargin{:});
else
    [m,n] = size(A);
    if( n > m )
        X = pinvSp(A',varargin{:})';
    else
        % computes smallest singular value of A:  
        %      (MATLAB's proposed svds(A,1,0) tends to not work)
        % sm = sqrt(eigs(@(x)A\(x'/A)',size(A,2),1,'sm'));
        % computes largest singular value of A:
        % sl = svds(A,1);
        [U,S,V] = svd(full(A),0);
        if( m > 1 )
            s = diag(S);
        elseif( m == 1 )
            s = S(1);
        else
            s = 0;
        end
        if( nargin == 2 )
            tol = varargin{1};
        else
            tol = max(m,n) * eps(max(s));
        end
        r = sum(s > tol);
        if( r == 0 )
            X = zeros(size(A'),class(A));
        else
            s = diag(ones(r,1)./s(1:r));
            X = V(:,1:r)*s*U(:,1:r)';
        end
    end
end